@@ -37,13 +37,21 @@
 #include <kdebug.h>
 
 // system includes
+#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(USE_SOLARIS)
+#include <sys/statfs.h>
+#elif defined(__solaris__) || defined(USE_SOLARIS)
+#include <sys/statvfs.h>
+#elif defined(__FreeBSD__)
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
 #include <sys/types.h>
+#include <pwd.h>
 #include <unistd.h>
 #include <errno.h>
 #include <dirent.h>
 
 #ifdef __FreeBSD__
-#include <pwd.h>
 #include <sys/param.h>
 #include <sys/ucred.h>
 #include <sys/mount.h>
@@ -85,9 +93,6 @@ Smb4KMounter::Smb4KMounter( QObject *parent, const char *name ) : QObject( paren
   connect( m_proc,  SIGNAL( receivedStderr( KProcess *, char *, int ) ),
            this,    SLOT( slotReceivedStderr( KProcess *, char *, int ) ) );
 
-  connect( timer(), SIGNAL( timeout() ),
-           this,    SLOT( slotProcessQueue() ) );
-
   connect( kapp,    SIGNAL( shutDown() ),
            this,    SLOT( slotShutdown() ) );
 }
@@ -112,6 +117,8 @@ void Smb4KMounter::init()
 {
   m_queue.enqueue( new QString( QString( "%1:" ).arg( Import ) ) );
   m_queue.enqueue( new QString( QString( "%1:" ).arg( Remount ) ) );
+
+  startTimer( TIMER_INTERVAL );
 }
 
 
@@ -128,32 +135,9 @@ void Smb4KMounter::abort()
   {
     if ( Smb4KSettings::alwaysUseSuperUser() )
     {
-      QString suid_program;
-
-      switch( Smb4KSettings::superUserProgram() )
-      {
-        case Smb4KSettings::EnumSuperUserProgram::Sudo:
-        {
-          suid_program = Smb4KSettings::sudo();
-
-          break;
-        }
-        case Smb4KSettings::EnumSuperUserProgram::Super:
-        {
-          suid_program = Smb4KSettings::super();
-
-          break;
-        }
-        default:
-        {
-          // FIXME: Throw an error?
-          return;
-        }
-      }
-
       KProcess proc;
       proc.setUseShell( true );
-      proc << QString( "%1 smb4k_kill %2" ).arg( suid_program ).arg( m_proc->pid() );
+      proc << QString( "%1 smb4k_kill %2" ).arg( Smb4KSettings::sudo() ).arg( m_proc->pid() );
       proc.start( KProcess::DontCare, KProcess::NoCommunication );
     }
     else
@@ -295,7 +279,7 @@ void Smb4KMounter::import()
 
         if ( (*it).contains( " smbfs ", false ) != 0 )
         {
-          QString share_and_path = (*it).section( "smbfs", 0, 0 ).stripWhiteSpace();
+          QString share_and_path = (*it).section( " smbfs ", 0, 0 ).stripWhiteSpace();
           QString name = share_and_path.section( " ", 0, 0 ).stripWhiteSpace().replace( "\\040", "\040" );
           QString path = share_and_path.section( " ", 1, 1 ).stripWhiteSpace();
 
@@ -312,7 +296,7 @@ void Smb4KMounter::import()
         }
         else if ( (*it).contains( " cifs ", false ) != 0 )
         {
-          QString share_and_path = (*it).section( "cifs", 0, 0 ).stripWhiteSpace();
+          QString share_and_path = (*it).section( " cifs ", 0, 0 ).stripWhiteSpace();
           QString name = share_and_path.section( " ", 0, 0 ).stripWhiteSpace().replace( "\\040", "\040" );
           QString path = share_and_path.section( " ", 1, 1 ).stripWhiteSpace();
 
@@ -569,29 +553,10 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
   delete dir;
 
   // Now we are prepared to mount the share:
-  QString command, suid_program;
+  QString command;
 
-  switch ( Smb4KSettings::superUserProgram() )
-  {
-    case Smb4KSettings::EnumSuperUserProgram::Sudo:
-    {
-      suid_program = "sudo";
-
-      break;
-    }
-    case Smb4KSettings::EnumSuperUserProgram::Super:
-    {
-      suid_program = "super";
-
-      break;
-    }
-    default:
-    {
-      return;
-    }
-  }
-
-  Smb4KAuthInfo *auth = passwordHandler()->readAuth( new Smb4KAuthInfo( m_priv->workgroup(), m_priv->host(), m_priv->share() ) );
+  Smb4KAuthInfo authInfo( m_priv->workgroup(), m_priv->host(), m_priv->share() );
+  (void) passwordHandler()->readAuth( &authInfo );
 
 #ifndef __FreeBSD__
 
@@ -618,7 +583,7 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
     case Smb4KSettings::EnumFilesystem::CIFS:
     {
       command.append( Smb4KSettings::alwaysUseSuperUser() ?   // FIXME: Check if suid program is installed
-                      QString( "%1 smb4k_mount -s -t cifs " ).arg( suid_program ) :
+                      QString( "%1 smb4k_mount -s -t cifs " ).arg( Smb4KSettings::sudo() ) :
                       "smb4k_mount -n -t cifs " );
 
       command.append( "-o " );
@@ -633,16 +598,16 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
                       QString( "ip=%1," ).arg( m_priv->ip() ) :
                       "" );
 
-      command.append( !auth->user().isEmpty() ?
-                      QString( "user=%1" ).arg( auth->user() ) :
+      command.append( !authInfo.user().isEmpty() ?
+                      QString( "user=%1" ).arg( authInfo.user() ) :
                       "guest" );
 
       command.append( " -- " );
 
       command.append( QString( "//'%1'/'%2' '%3'" ).arg( m_priv->host(), m_priv->share(), m_priv->path() ) );
 
-      m_priv->setCIFSLogin( !auth->user().isEmpty() ?
-                      auth->user() :
+      m_priv->setCIFSLogin( !authInfo.user().isEmpty() ?
+                      authInfo.user() :
                       "guest" );
 
       m_priv->setFileSystem( "cifs" );
@@ -652,7 +617,7 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
     case Smb4KSettings::EnumFilesystem::SMBFS:
     {
       command.append( Smb4KSettings::alwaysUseSuperUser() ?   // FIXME: Check if suid program is installed
-                      QString( "%1 smb4k_mount -s -t smbfs " ).arg( suid_program ) :
+                      QString( "%1 smb4k_mount -s -t smbfs " ).arg( Smb4KSettings::sudo() ) :
                       "smb4k_mount -n -t smbfs " );
 
       command.append( "-o " );
@@ -667,8 +632,8 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
                       QString( "ip=%1," ).arg( m_priv->ip() ) :
                       "" );
 
-      command.append( !auth->user().isEmpty() ?
-                      QString( "username=%1" ).arg( auth->user() ) :
+      command.append( !authInfo.user().isEmpty() ?
+                      QString( "username=%1" ).arg( authInfo.user() ) :
                       "guest" );
 
       command.append( " -- " );
@@ -685,7 +650,7 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
     }
   }
 
-  m_proc->setEnvironment( "PASSWD", !auth->password().isEmpty() ? auth->password() : "" );
+  m_proc->setEnvironment( "PASSWD", !authInfo.password().isEmpty() ? authInfo.password() : "" );
 
 #else
 
@@ -696,7 +661,7 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
              Smb4KSettings::remotePort();
 
   command.append( Smb4KSettings::alwaysUseSuperUser() ?   // FIXME: Check if suid program is installed
-                  QString( "%1 smb4k_mount " ).arg( suid_program ) :
+                  QString( "%1 smb4k_mount " ).arg( Smb4KSettings::sudo() ) :
                   "smb4k_mount " );
 
   command.append( optionsHandler()->mountOptions( QString( "//%1/%2" ).arg( m_priv->host(), m_priv->share() ) ) );
@@ -713,15 +678,13 @@ void Smb4KMounter::mount( const QString &workgroup, const QString &host, const Q
 
   command.append( " -- " );
 
-  command.append( QString( "//%1@'%2':%3/'%4' '%5'" ).arg( !auth->user().isEmpty() ? auth->user() : "guest" )
+  command.append( QString( "//%1@'%2':%3/'%4' '%5'" ).arg( !authInfo.user().isEmpty() ? authInfo.user() : "guest" )
                                                      .arg( m_priv->host() )
                                                      .arg( port )
                                                      .arg( m_priv->share(), m_priv->path() ) );
 
 #endif
 
-  delete auth;
-
   // Start the mount process:
   *m_proc << command;
 
@@ -772,31 +735,7 @@ void Smb4KMounter::unmount( const QString &mountpoint, bool force, bool noMessag
     QString path = mountpoint;
     m_priv->setPath( path.replace( '\044', "\044" ) );
 
-    QString suid_program, command;
-
-    if ( Smb4KSettings::useForceUnmount() || Smb4KSettings::alwaysUseSuperUser() )
-    {
-      switch ( Smb4KSettings::superUserProgram() )
-      {
-        case Smb4KSettings::EnumSuperUserProgram::Sudo:
-        {
-          suid_program = Smb4KSettings::sudo();
-
-          break;
-        }
-        case Smb4KSettings::EnumSuperUserProgram::Super:
-        {
-          suid_program = Smb4KSettings::super();
-
-          break;
-        }
-        default:
-        {
-          // FIXME: Throw an error?
-          return;
-        }
-      }
-    }
+    QString command;
 
     Smb4KShare *share = findShareByPath( mountpoint );
 
@@ -809,12 +748,12 @@ void Smb4KMounter::unmount( const QString &mountpoint, bool force, bool noMessag
           if ( KMessageBox::questionYesNo( 0, i18n( "Do you really want to force the unmounting of this share?" ), QString::null, KStdGuiItem::yes(), KStdGuiItem::no(), "Dont Ask Forced", KMessageBox::Notify ) == KMessageBox::Yes )
           {
 #ifdef __linux__
-            command.append( QString( "%1 smb4k_umount -s -l " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount -s -l " ).arg( Smb4KSettings::sudo() ) );
 #else
 #ifdef __FreeBSD__
-            command.append( QString( "%1 smb4k_umount " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount " ).arg( Smb4KSettings::sudo() ) );
 #else
-            command.append( QString( "%1 smb4k_umount -s " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount -s " ).arg( Smb4KSettings::sudo() ) );
 #endif
 #endif
             execute = true;
@@ -840,9 +779,9 @@ void Smb4KMounter::unmount( const QString &mountpoint, bool force, bool noMessag
           else
           {
 #ifndef __FreeBSD__
-            command.append( QString( "%1 smb4k_umount -s " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount -s " ).arg( Smb4KSettings::sudo() ) );
 #else
-            command.append( QString( "%1 smb4k_umount " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount " ).arg( Smb4KSettings::sudo() ) );
 #endif
           }
         }
@@ -856,12 +795,12 @@ void Smb4KMounter::unmount( const QString &mountpoint, bool force, bool noMessag
             if ( KMessageBox::questionYesNo( 0, i18n( "Do you really want to force the unmounting of this share?" ), QString::null, KStdGuiItem::yes(), KStdGuiItem::no(), "Dont Ask Forced", KMessageBox::Notify ) == KMessageBox::Yes )
             {
 #ifdef __linux__
-            command.append( QString( "%1 smb4k_umount -s -l " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount -s -l " ).arg( Smb4KSettings::sudo() ) );
 #else
 #ifdef __FreeBSD__
-            command.append( QString( "%1 smb4k_umount " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount " ).arg( Smb4KSettings::sudo() ) );
 #else
-            command.append( QString( "%1 smb4k_umount -s " ).arg( suid_program ) );
+            command.append( QString( "%1 smb4k_umount -s " ).arg( Smb4KSettings::sudo() ) );
 #endif
 #endif
               execute = true;
@@ -887,9 +826,9 @@ void Smb4KMounter::unmount( const QString &mountpoint, bool force, bool noMessag
             else
             {
 #ifndef __FreeBSD__
-              command.append( QString( "%1 smb4k_umount -s " ).arg( suid_program ) );
+              command.append( QString( "%1 smb4k_umount -s " ).arg( Smb4KSettings::sudo() ) );
 #else
-              command.append( QString( "%1 smb4k_umount " ).arg( suid_program ) );
+              command.append( QString( "%1 smb4k_umount " ).arg( Smb4KSettings::sudo() ) );
 #endif
             }
           }
@@ -1039,21 +978,102 @@ void Smb4KMounter::processMount()
     {
       QString name = QString( "//%1/%2" ).arg( m_priv->host() ).arg( m_priv->share() );
 
-      if ( QString::compare( m_priv->filesystem(), "smbfs" ) == 0 )
+      // Check file system
+#if !defined(__solaris__)
+      struct statfs filesystem;
+#else
+      struct statvfs filesystem;
+#endif
+
+#if !defined(__solaris__) && !defined(__irix__)
+      if ( statfs( m_priv->path(), &filesystem ) == -1 )
+#elif defined(__irix__)
+      if ( statfs( m_priv->path(), &filesystem, sizeof( filesystem ), 0 ) == -1 )
+#else
+      if ( statvfs( m_priv->path(), &filesystem ) == -1 )
+#endif
       {
-        share =  new Smb4KShare( name, m_priv->path(), m_priv->filesystem(), (int)getuid(), (int)getgid() );
-        m_mounted_shares.append( share );
+        // The query failed. Go with the file system already defined in m_priv.
+        if ( QString::compare( m_priv->filesystem(), "smbfs" ) == 0 )
+        {
+          share =  new Smb4KShare( name, m_priv->path(), m_priv->filesystem(), (int)getuid(), (int)getgid() );
+          m_mounted_shares.append( share );
+        }
+        else if ( QString::compare( m_priv->filesystem(), "cifs" ) == 0 )
+        {
+          // The user name will be send if no login was specified.
+          QString cifs_login = !m_priv->cifsLogin().isEmpty() ?
+                               m_priv->cifsLogin() :
+                               getpwuid( getuid() )->pw_name;
+
+          share = new Smb4KShare( name, m_priv->path(), m_priv->filesystem(), cifs_login, false );
+          m_mounted_shares.append( share );
+        }
       }
-      else if ( QString::compare( m_priv->filesystem(), "cifs" ) == 0 )
+      else
       {
-        share = new Smb4KShare( name, m_priv->path(), m_priv->filesystem(), m_priv->cifsLogin(), false );
-        m_mounted_shares.append( share );
+#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(__irix__)
+        if ( (uint)filesystem.f_type == 0xFF534D42 /* CIFS */)
+        {
+          // The user name will be send if no login was specified.
+          QString cifs_login = !m_priv->cifsLogin().isEmpty() ?
+                               m_priv->cifsLogin() :
+                               getpwuid( getuid() )->pw_name;
+
+          share = new Smb4KShare( name, m_priv->path(), "cifs", cifs_login, false );
+          m_mounted_shares.append( share );
+        }
+        else if ( (uint)filesystem.f_type == 0x517B /* SMBFS */)
+        {
+          share =  new Smb4KShare( name, m_priv->path(), "smbfs", (int)getuid(), (int)getgid() );
+          m_mounted_shares.append( share );
+        }
+#elif defined(__solaris__)
+        if ( (uint)filesystem.f_basetype == 0xFF534D42 /* CIFS */)
+        {
+          // The user name will be send if no login was specified.
+          QString cifs_login = !m_priv->cifsLogin().isEmpty() ?
+                               m_priv->cifsLogin() :
+                               getpwuid( getuid() )->pw_name;
+
+          share = new Smb4KShare( name, m_priv->path(), "cifs", cifs_login, false );
+          m_mounted_shares.append( share );
+        }
+        else if ( (uint)filesystem.f_basetype == 0x517B /* SMBFS */)
+        {
+          share =  new Smb4KShare( name, m_priv->path(), "smbfs", (int)getuid(), (int)getgid() );
+          m_mounted_shares.append( share );
+        }
+#elif defined(__irix__)
+        if ( (uint)filesystem.f_fstyp == 0xFF534D42 /* CIFS */)
+        {
+          // The user name will be send if no login was specified.
+          QString cifs_login = !m_priv->cifsLogin().isEmpty() ?
+                               m_priv->cifsLogin() :
+                               getpwuid( getuid() )->pw_name;
+
+          share = new Smb4KShare( name, m_priv->path(), "cifs", cifs_login, false );
+          m_mounted_shares.append( share );
+        }
+        else if ( (uint)filesystem.f_basetype == 0x517B && !strncmp( fs, "smbfs", strlen( fs )+1 ) )
+        {
+          share =  new Smb4KShare( name, m_priv->path(), "smbfs", (int)getuid(), (int)getgid() );
+          m_mounted_shares.append( share );
+        }
+#endif
+        else
+        {
+          // Error... We don't create a share.
+        }
       }
 
-      // Check that the share is accessible:
-      checkAccessibility( share );
+      if ( share )
+      {
+        // Check that the share is accessible:
+        checkAccessibility( share );
 
-      emit mountedShare( m_priv->path() );
+        emit mountedShare( m_priv->path() );
+      }
     }
     else
     {
@@ -1110,11 +1130,10 @@ void Smb4KMounter::processMount()
     {
       import();  // FIXME: *cough* What is this for???
 
-      Smb4KAuthInfo *auth = passwordHandler()->readAuth( new Smb4KAuthInfo( m_priv->workgroup(), m_priv->host(), m_priv->share() ) );
+      Smb4KAuthInfo authInfo( m_priv->workgroup(), m_priv->host(), m_priv->share() );
+      (void) passwordHandler()->readAuth( &authInfo );
 
-      QString name = QString( "//%1@%2/%3" ).arg( auth->user().upper(), m_priv->host().upper(), m_priv->share().upper() );
-
-      delete auth;
+      QString name = QString( "//%1@%2/%3" ).arg( authInfo.user().upper(), m_priv->host().upper(), m_priv->share().upper() );
 
       share = new Smb4KShare( name, m_priv->path(), m_priv->filesystem(), (int)getuid(), (int)getgid() );
       m_mounted_shares.append( share );
@@ -1136,11 +1155,10 @@ void Smb4KMounter::processMount()
       }
       else
       {
-        Smb4KAuthInfo *auth = passwordHandler()->readAuth( new Smb4KAuthInfo( m_priv->workgroup(), m_priv->host(), m_priv->share() ) );
+        Smb4KAuthInfo authInfo(  m_priv->workgroup(), m_priv->host(), m_priv->share() );
+        (void) passwordHandler()->readAuth( &authInfo );
 
-        QString name = QString( "//%1@%2/%3" ).arg( auth->user().upper(), m_priv->host().upper(), m_priv->share().upper() );
-
-        delete auth;
+        QString name = QString( "//%1@%2/%3" ).arg( authInfo.user().upper(), m_priv->host().upper(), m_priv->share().upper() );
 
         Smb4KError::error( ERROR_MOUNTING_SHARE, name, m_buffer );
       }
@@ -1389,30 +1407,7 @@ void Smb4KMounter::checkAccessibility( Smb4KShare *share )
 }
 
 
-/////////////////////////////////////////////////////////////////////////////
-// SLOT IMPLEMENTATIONS
-/////////////////////////////////////////////////////////////////////////////
-
-
-void Smb4KMounter::slotProcessExited( KProcess * )
-{
-  endProcess();
-}
-
-
-void Smb4KMounter::slotReceivedStdout( KProcess *, char *buf, int len )
-{
-  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
-}
-
-
-void Smb4KMounter::slotReceivedStderr( KProcess *, char *buf, int len )
-{
-  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
-}
-
-
-void Smb4KMounter::slotProcessQueue()
+void Smb4KMounter::timerEvent( QTimerEvent * )
 {
   if ( !m_working && !m_queue.isEmpty() )
   {
@@ -1471,6 +1466,29 @@ void Smb4KMounter::slotProcessQueue()
 }
 
 
+/////////////////////////////////////////////////////////////////////////////
+// SLOT IMPLEMENTATIONS
+/////////////////////////////////////////////////////////////////////////////
+
+
+void Smb4KMounter::slotProcessExited( KProcess * )
+{
+  endProcess();
+}
+
+
+void Smb4KMounter::slotReceivedStdout( KProcess *, char *buf, int len )
+{
+  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
+}
+
+
+void Smb4KMounter::slotReceivedStderr( KProcess *, char *buf, int len )
+{
+  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
+}
+
+
 void Smb4KMounter::slotShutdown()
 {
   // Abort any action:
@@ -1544,28 +1562,7 @@ void Smb4KMounter::slotShutdown()
 
   if ( Smb4KSettings::unmountSharesOnExit() )
   {
-    QString suid_program, command;
-
-    switch( Smb4KSettings::superUserProgram() )
-    {
-      case Smb4KSettings::EnumSuperUserProgram::Sudo:
-      {
-        suid_program = Smb4KSettings::sudo();
-
-        break;
-      }
-      case Smb4KSettings::EnumSuperUserProgram::Super:
-      {
-        suid_program = Smb4KSettings::super();
-
-        break;
-      }
-      default:
-      {
-        // FIXME: Throw an error?
-        return;
-      }
-    }
+    QString command;
 
     KProcess proc;
     proc.setUseShell( true );
@@ -1578,9 +1575,9 @@ void Smb4KMounter::slotShutdown()
         if ( Smb4KSettings::alwaysUseSuperUser() )
         {
 #ifndef __FreeBSD__
-          command.append( QString( "%1 smb4k_umount -s -t %2 " ).arg( suid_program ).arg( (*it)->filesystem() ) );
+          command.append( QString( "%1 smb4k_umount -s -t %2 " ).arg( Smb4KSettings::sudo() ).arg( (*it)->filesystem() ) );
 #else
-          command.append( QString( "%1 smb4k_umount " ).arg( suid_program ) );
+          command.append( QString( "%1 smb4k_umount " ).arg( Smb4KSettings::sudo() ) );
 #endif
           command.append( KProcess::quote( (*it)->path() ) );
           command.append( " ; " );
