@@ -143,14 +143,13 @@ Smb4KScanner::~Smb4KScanner()
 
 void Smb4KScanner::init()
 {
+  m_timer_id = startTimer( TIMER_INTERVAL );
   rescan();
 }
 
 
 void Smb4KScanner::rescan()
 {
-  connect_timer( true );
-
   m_queue.enqueue( new QString( QString( "%1:" ).arg( Init ) ) );
 }
 
@@ -161,8 +160,6 @@ void Smb4KScanner::rescan()
 
 void Smb4KScanner::getWorkgroupMembers( const QString &workgroup, const QString &master, const QString &ip )
 {
-  connect_timer( true );
-
   m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Hosts ).arg( workgroup, master, ip ) ) );
 }
 
@@ -173,8 +170,6 @@ void Smb4KScanner::getWorkgroupMembers( const QString &workgroup, const QString
 
 void Smb4KScanner::getShares( const QString &workgroup, const QString &host, const QString &ip, const QString &protocol )
 {
-  connect_timer( true );
-
   m_queue.enqueue( new QString( QString( "%1:%2:%3:%4:%5" ).arg( Shares ).arg( workgroup, host, ip ).arg( protocol ) ) );
 }
 
@@ -199,8 +194,6 @@ void Smb4KScanner::getInfo( const QString &workgroup, const QString &host, const
     item->setInfoChecked( true );
   }
 
-  connect_timer( true );
-
   m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Info ).arg( workgroup, host, ip ) ) );
 }
 
@@ -213,17 +206,22 @@ void Smb4KScanner::search( const QString &host )
 {
   // Check whether we already have this host in
   // the list:
-  Smb4KHostItem *item = getHost( host );
-
-  if ( item )
+  if ( !host.isEmpty() )
   {
-    emit searchResult( item );
+    Smb4KHostItem *item = getHost( host );
 
+    if ( item )
+    {
+      emit searchResult( item );
+
+      return;
+    }
+  }
+  else
+  {
     return;
   }
 
-  connect_timer( true );
-
   m_queue.enqueue( new QString( QString( "%1:%2" ).arg( Search ).arg( host ) ) );
 }
 
@@ -389,9 +387,34 @@ void Smb4KScanner::scanForWorkgroupMembers( const QString &workgroup, const QStr
     command.append( " -I "+ip );
     command.append( " -w "+KProcess::quote( workgroup ) );
     command.append( " -S "+KProcess::quote( master ) );
-    // NOTE: Do not set the environment variables USER and PASSWD
-    // here, because this won't work at all (Why?).
-    command.append( " -U %" );
+
+    if ( Smb4KSettings::masterBrowsersRequireAuth() )
+    {
+      Smb4KAuthInfo authInfo( workgroup, master, QString::null );
+      (void) passwordHandler()->readAuth( &authInfo );
+
+      if ( !authInfo.user().isEmpty() )
+      {
+        command.append( QString( " -U %1" ).arg( KProcess::quote( authInfo.user() ) ) );
+
+        if ( !authInfo.password().isEmpty() )
+        {
+          m_proc->setEnvironment( "PASSWD", authInfo.password() );
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        command.append( " -U %" );
+      }
+    }
+    else
+    {
+      command.append( " -U %" );
+    }
   }
   else
   {
@@ -406,9 +429,34 @@ void Smb4KScanner::scanForWorkgroupMembers( const QString &workgroup, const QStr
     command.append( " -I IPADDR" );
     command.append( " -w "+KProcess::quote( workgroup ) );
     command.append( " -S "+KProcess::quote( master ) );
-    // NOTE: Do not set the environment variables USER and PASSWD
-    // here, because this won't work at all (Why?).
-    command.append( " -U %'" );
+
+    if ( Smb4KSettings::masterBrowsersRequireAuth() )
+    {
+      Smb4KAuthInfo authInfo( workgroup, master, QString::null );
+      (void) passwordHandler()->readAuth( &authInfo );
+
+      if ( !authInfo.user().isEmpty() )
+      {
+        command.append( QString( " -U %1'" ).arg( KProcess::quote( authInfo.user() ) ) );
+
+        if ( !authInfo.password().isEmpty() )
+        {
+          m_proc->setEnvironment( "PASSWD", authInfo.password() );
+        }
+        else
+        {
+          // Do nothing
+        }
+      }
+      else
+      {
+        command.append( " -U %'" );
+      }
+    }
+    else
+    {
+      command.append( " -U %'" );
+    }
   }
 
   *m_proc << command;
@@ -633,11 +681,6 @@ void Smb4KScanner::endProcess()
 
   m_proc->clearArguments();
 
-  if ( m_queue.isEmpty() )
-  {
-    connect_timer( false );
-  }
-
   m_working = false;
 
   emit state( SCANNER_STOP );
@@ -883,10 +926,25 @@ void Smb4KScanner::processWorkgroupMembers()
     case Smb4KSettings::EnumBrowseList::QueryCurrentMaster:
     case Smb4KSettings::EnumBrowseList::QueryCustomMaster:
     {
-      if ( (m_buffer.contains( "Could not connect to server" ) != 0 &&
-           m_buffer.contains( "The username or password was not correct" ) == 0) ||
-           m_buffer.contains( "Unable to find a suitable server" ) != 0 ||
-           m_buffer.contains( "Invalid ip address specified" ) != 0 )
+      if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 ||
+           m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0 ||
+           m_buffer.contains( "The username or password was not correct" ) != 0 )
+      {
+        // Authentication failed:
+        emit failed();
+
+        if ( passwordHandler()->askpass( m_priv->workgroup(), m_priv->host(),
+             QString::null, Smb4KPasswordHandler::AccessDenied,
+             kapp->mainWidget() ? kapp->mainWidget() : 0, "AskPass" ) )
+        {
+           m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Hosts ).arg( m_priv->workgroup(), m_priv->host(), m_priv->ip() ) ) );
+        }
+
+        return;
+      }
+      else if ( m_buffer.contains( "Could not connect to server" ) != 0 ||
+                m_buffer.contains( "Unable to find a suitable server" ) != 0 ||
+                m_buffer.contains( "Invalid ip address specified" ) != 0 )
       {
         // If the IP address is empty, the shell output contains
         // the IP address. Remove it, because it will confuse
@@ -904,6 +962,10 @@ void Smb4KScanner::processWorkgroupMembers()
 
         return;
       }
+      else
+      {
+        // Do nothing
+      }
 
       QValueList<Smb4KHostItem *> hosts;
 
@@ -1046,10 +1108,25 @@ void Smb4KScanner::processWorkgroupMembers()
     }
     case Smb4KSettings::EnumBrowseList::ScanBroadcastAreas:
     {
-      if ( (m_buffer.contains( "Could not connect to server" ) != 0 &&
-            m_buffer.contains( "The username or password was not correct" ) == 0) ||
-          m_buffer.contains( "Unable to find a suitable server" ) != 0 ||
-          m_buffer.contains( "Invalid ip address specified" ) != 0 )
+      if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 ||
+           m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0 ||
+           m_buffer.contains( "The username or password was not correct" ) != 0 )
+      {
+        // Authentication failed:
+        emit failed();
+
+        if ( passwordHandler()->askpass( m_priv->workgroup(), m_priv->host(),
+             QString::null, Smb4KPasswordHandler::AccessDenied,
+             kapp->mainWidget() ? kapp->mainWidget() : 0, "AskPass" ) )
+        {
+           m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Hosts ).arg( m_priv->workgroup(), m_priv->host(), m_priv->ip() ) ) );
+        }
+
+        return;
+      }
+      else if ( m_buffer.contains( "Could not connect to server" ) != 0 ||
+                m_buffer.contains( "Unable to find a suitable server" ) != 0 ||
+                m_buffer.contains( "Invalid ip address specified" ) != 0 )
       {
         // We are in IP scan mode, so we can ignore the error and emit
         // what we already have:
@@ -1409,23 +1486,6 @@ Smb4KHostItem *Smb4KScanner::getHost( const QString &name, const QString &workgr
 }
 
 
-/****************************************************************************
-   Connect or disconnect the timer
-****************************************************************************/
-
-void Smb4KScanner::connect_timer( bool con )
-{
-  if ( con )
-  {
-    connect( timer(), SIGNAL( timeout() ), this, SLOT( slotProcessQueue() ) );
-  }
-  else
-  {
-    disconnect( timer(), SIGNAL( timeout() ), this, SLOT( slotProcessQueue() ) );
-  }
-}
-
-
 /****************************************************************************
    Add a host to the host list
 ****************************************************************************/
@@ -1480,34 +1540,7 @@ void Smb4KScanner::appendWorkgroup( Smb4KWorkgroupItem *item )
 }
 
 
-/////////////////////////////////////////////////////////////////////////////
-// SLOT IMPLEMENTATIONS
-/////////////////////////////////////////////////////////////////////////////
-
-
-/****************************************************************************
-   Internal slots.
-****************************************************************************/
-
-void Smb4KScanner::slotReceivedStdout( KProcess *, char *buf, int len )
-{
-  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
-}
-
-
-void Smb4KScanner::slotProcessExited( KProcess * )
-{
-  endProcess();
-}
-
-
-void Smb4KScanner::slotReceivedStderr( KProcess *, char  *buf, int len )
-{
-  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
-}
-
-
-void Smb4KScanner::slotProcessQueue()
+void Smb4KScanner::timerEvent( QTimerEvent * )
 {
   // Look for the thing to do (next).
   // At this point, the topmost item will not be
@@ -1577,6 +1610,33 @@ void Smb4KScanner::slotProcessQueue()
 }
 
 
+/////////////////////////////////////////////////////////////////////////////
+// SLOT IMPLEMENTATIONS
+/////////////////////////////////////////////////////////////////////////////
+
+
+/****************************************************************************
+   Internal slots.
+****************************************************************************/
+
+void Smb4KScanner::slotReceivedStdout( KProcess *, char *buf, int len )
+{
+  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
+}
+
+
+void Smb4KScanner::slotProcessExited( KProcess * )
+{
+  endProcess();
+}
+
+
+void Smb4KScanner::slotReceivedStderr( KProcess *, char  *buf, int len )
+{
+  m_buffer.append( QString::fromLocal8Bit( buf, len ) );
+}
+
+
 
 /***************************************************************************
  *                                                                         *
