@@ -156,14 +156,10 @@ void Smb4KApp::setupActions()
   KActionMenu *view_modes  = new KActionMenu( i18n( "Shares Vie&w" ), "view_choose",
                              actionCollection(), "view_modes_menu" );
 
-//   KRadioAction *icon_view  = new KRadioAction( i18n( "&Icon View" ), "view_icon", CTRL+Key_I,
-//                              this, SLOT( slotSharesIconView() ), actionCollection(), "icon_view_action" );
   KRadioAction *icon_view  = new KRadioAction( i18n( "&Icon View" ), "view_icon", CTRL+Key_I,
                              this, SLOT( slotChangeSharesView() ), actionCollection(), "icon_view_action" );
   icon_view->setExclusiveGroup( "SharesViewActions" );
 
-//   KRadioAction *list_view  = new KRadioAction( i18n( "List Vie&w" ), "view_detailed", CTRL+Key_W,
-//                              this, SLOT( slotSharesListView() ), actionCollection(), "list_view_action" );
   KRadioAction *list_view  = new KRadioAction( i18n( "List Vie&w" ), "view_detailed", CTRL+Key_W,
                              this, SLOT( slotChangeSharesView() ), actionCollection(), "list_view_action" );
   list_view->setExclusiveGroup( "SharesViewActions" );
@@ -480,6 +476,17 @@ bool Smb4KApp::queryClose()
 }
 
 
+void Smb4KApp::timerEvent( QTimerEvent * )
+{
+  KProgress *progress_bar = static_cast<KProgress *>( child( "StatusBarProgressBar", "KProgress", true ) );
+
+  if ( progress_bar )
+  {
+    progress_bar->setProgress( progress_bar->progress() + 1 );
+  }
+}
+
+
 /////////////////////////////////////////////////////////////////////
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////
@@ -622,17 +629,6 @@ void Smb4KApp::slotBookmarkEditor()
 }
 
 
-void Smb4KApp::slotSetProgress()
-{
-  KProgress *progress_bar = static_cast<KProgress *>( child( "StatusBarProgressBar", "KProgress", true ) );
-
-  if ( progress_bar )
-  {
-    progress_bar->setProgress( progress_bar->progress() + 1 );
-  }
-}
-
-
 void Smb4KApp::slotRunStateChanged()
 {
   // Get the progress bar:
@@ -792,16 +788,14 @@ void Smb4KApp::slotRunStateChanged()
     {
       progress_bar->setTotalSteps( 0 );
 
-      connect( Smb4KGlobal::timer(), SIGNAL( timeout() ),
-               this,                 SLOT( slotSetProgress() ) );
+      m_timer_id = startTimer( TIMER_INTERVAL );
     }
   }
   else
   {
     if ( progress_bar )
     {
-      disconnect( Smb4KGlobal::timer(), SIGNAL( timeout() ),
-                  this,                 SLOT( slotSetProgress() ) );
+      killTimer( m_timer_id );
 
       progress_bar->setTotalSteps( 100 );
       progress_bar->reset();
